import nfshttpclient
import os
import json
import hashlib
from nfslogger import logger

# global 
LOCAL_BOOKMARK_HASH = 0
LOCAL_LOGINDATA_HASH = 0
def run(username):
    global LOCAL_BOOKMARK_HASH
    global LOCAL_LOGINDATA_HASH
    token = nfshttpclient.getUserToken(username)
    if token:
        if nfshttpclient.is_sync_feature_enabled(username):
            local_chrome_json_file = nfshttpclient.get_chrome_config_file(username)
            chrom_json_file = open(local_chrome_json_file, 'r')
            config = chrom_json_file.read()
            chrom_json_file.close()
            config =  config.strip('\n')
            config_json = json.loads(config)
            #"{\"syncAllDataTypes\":true,\"bookmarks\":false,\"passwords\":true}"
            syncItemSwitchConfig_json = json.loads(config_json['syncItemSwitchConfig'])
            syncAllDataTypes = syncItemSwitchConfig_json['syncAllDataTypes']
            sync_bookmarks = syncItemSwitchConfig_json['bookmarks']
            sync_passwords = syncItemSwitchConfig_json['passwords']
            if syncAllDataTypes is True:
                logger.debug("syncAllDataTypes is true ,compare all")
                compareBookmark(username)
                comparePassword(username)
                return
            if sync_bookmarks is True:
                logger.debug("sync_bookmarks is true,begin compare bookmark")
                compareBookmark(username)
            if sync_passwords is True:
                logger.debug("sync_passwords is true,begin compare  Password")
                comparePassword(username)
    else:
        LOCAL_BOOKMARK_HASH = 0
        LOCAL_LOGINDATA_HASH = 0
        logger.debug("reset  LOCAL_BOOKMARK_HASH and LOCAL_LOGINDATA_HASH")
        return ""

def get_chrom_dir(username):
    #  return '/home/'+username+'/.config/chromium/Default/'
    return '/home/'+username+'/.config/nfs-browser/Default/'
  

def get_caccount_dir(username):
    return '/home/'+username+'/.caccount/config/'

#get file hash
def hash_file(filename):
    hasher = hashlib.md5()
    with open(filename, 'rb') as f:
        buf = f.read()
        hasher.update(buf)
    return hasher.hexdigest()

def compareBookmark(username):
    global LOCAL_BOOKMARK_HASH
    
    chrom_dir = get_chrom_dir(username)
    caccount_dir = get_caccount_dir(username)
    bookmakr_file_name = "Bookmarks"
    server_bookmark = caccount_dir+bookmakr_file_name
    chrome_bookmark = chrom_dir+bookmakr_file_name
    
    #下载服务器最新的书签文件
    nfshttpclient.download(username,bookmakr_file_name,server_bookmark)
     
    if os.path.isfile(server_bookmark) is False:
        logger.info("download bookmark file form server error")
        return
    logger.info("download bookmark file form server ok")
    chown_cmd="chown "+username+":"+username+"  "+server_bookmark
    os.popen(chown_cmd)
    server_file_size = os.path.getsize(server_bookmark)
    #如果务器最新的书签文件是空的 
    if server_file_size == 0:
        logger.info("server bookmakr file is null")
        if os.path.isfile(chrome_bookmark) :
            nfshttpclient.upload(username,bookmakr_file_name,chrome_bookmark)
            logger.info("upload bookmark file to server")
            LOCAL_BOOKMARK_HASH = hash_file(chrome_bookmark)
        return
    logger.info("server bookmark file is not null")
    server_file_hash = hash_file(server_bookmark)
    if os.path.isfile(chrome_bookmark)  :
        chrome_file_hash = hash_file(chrome_bookmark)
        if LOCAL_BOOKMARK_HASH == 0:
            logger.debug('first local compare,ignore local,apply server file to local')
            mv_cmd="mv "+chrom_dir+bookmakr_file_name+" "+chrom_dir+bookmakr_file_name+".bk"
            logger.debug("mv_cmd "+mv_cmd)
            os.popen(mv_cmd)

            cp_cmd="cp "+server_bookmark+" "+chrom_dir
            logger.debug("cp_cmd "+cp_cmd)
            os.popen(cp_cmd)

            chown_cmd="chown "+username+":"+username+"  "+chrom_dir+bookmakr_file_name
            logger.debug("chown_cmd "+chown_cmd)
            os.popen(chown_cmd) 
            LOCAL_BOOKMARK_HASH = chrome_file_hash
        else:
            if LOCAL_BOOKMARK_HASH != chrome_file_hash:
                LOCAL_BOOKMARK_HASH = chrome_file_hash
                logger.info('local bookmark changed,upload it to server and return')
                nfshttpclient.upload(username,bookmakr_file_name,chrome_bookmark)
            else:
                logger.info('local bookmark not changed,compare to  server file')
                if chrome_file_hash != server_file_hash:
                    logger.debug("server bookmark file have changed,cp it to local")
                    mv_cmd="mv "+chrom_dir+bookmakr_file_name+" "+chrom_dir+bookmakr_file_name+".bk"
                    logger.debug("mv_cmd "+mv_cmd)
                    os.popen(mv_cmd)

                    cp_cmd="cp "+server_bookmark+" "+chrom_dir
                    logger.debug("cp_cmd "+cp_cmd)
                    os.popen(cp_cmd)

                    chown_cmd="chown "+username+":"+username+"  "+chrom_dir+bookmakr_file_name
                    logger.debug("chown_cmd "+chown_cmd)
                    os.popen(chown_cmd)
                else:
                    logger.debug("server bookmark equals local,do nothing")
    else:
        logger.debug("no local bookmark file,apply server bookmark to local")
        mv_cmd="mv "+chrom_dir+bookmakr_file_name+" "+chrom_dir+bookmakr_file_name+".bk"
        logger.debug("mv_cmd "+mv_cmd)
        os.popen(mv_cmd)

        cp_cmd="cp "+server_bookmark+" "+chrom_dir
        logger.debug("cp_cmd "+cp_cmd)
        os.popen(cp_cmd)

        chown_cmd="chown "+username+":"+username+"  "+chrom_dir+bookmakr_file_name
        logger.debug("chown_cmd "+chown_cmd)
        os.popen(chown_cmd)    
    
    return username


def comparePassword(username):
    global LOCAL_LOGINDATA_HASH
    chrom_dir = get_chrom_dir(username)
    caccount_dir = get_caccount_dir(username)
    logindata_file_name = "Login Data"
    logindata_file_in_chrome_name = "Login\ Data"
    server_logindata = caccount_dir+logindata_file_name
    chrome_logindata = chrom_dir+logindata_file_name
    
    
    #下载服务器最新的文件
    nfshttpclient.download(username,logindata_file_name,server_logindata)
     
    if os.path.isfile(server_logindata) is False:
        logger.info("download logindata file form server error")
        return
    logger.info("download logindata file form server ok")
    server_file_size = os.path.getsize(server_logindata)
    #如果务器最新的文件是空的 
    if server_file_size == 0:
        logger.info("server logindata file is null")
        if os.path.isfile(chrome_logindata) :
            nfshttpclient.upload(username,logindata_file_name,chrome_logindata)
            logger.info("upload logindata file to server")
            LOCAL_LOGINDATA_HASH = hash_file(chrome_logindata)
        return
    logger.info("server logindata file is not null")
    chown_cmd="chown "+username+":"+username+"  "+caccount_dir+logindata_file_in_chrome_name
    os.popen(chown_cmd)
    server_file_hash = hash_file(server_logindata)
    if os.path.isfile(chrome_logindata)  :
        chrome_file_hash = hash_file(chrome_logindata)
        if LOCAL_LOGINDATA_HASH == 0:
            logger.debug('first local compare,ignore local,apply server file to local')
            mv_cmd="mv "+chrom_dir+logindata_file_in_chrome_name+" "+chrom_dir+logindata_file_in_chrome_name+".bk"
            logger.debug("mv_cmd "+mv_cmd)
            os.popen(mv_cmd)

            cp_cmd="cp "+caccount_dir+logindata_file_in_chrome_name+" "+chrom_dir
            logger.debug("cp_cmd "+cp_cmd)
            os.popen(cp_cmd)

            chown_cmd="chown "+username+":"+username+"  "+chrom_dir+logindata_file_in_chrome_name
            logger.debug("chown_cmd "+chown_cmd)
            os.popen(chown_cmd)
            LOCAL_LOGINDATA_HASH = chrome_file_hash
        else:
            if LOCAL_LOGINDATA_HASH != chrome_file_hash:
                LOCAL_LOGINDATA_HASH = chrome_file_hash
                logger.info('local logindata changed,upload it to server and return')
                nfshttpclient.upload(username,logindata_file_name,chrome_logindata)
            else:
                logger.info('local logindata not changed,compare to  server file')
                if chrome_file_hash != server_file_hash:
                    logger.debug("server logindata file have changed,cp it to local")
                    mv_cmd="mv "+chrom_dir+logindata_file_in_chrome_name+" "+chrom_dir+logindata_file_in_chrome_name+".bk"
                    logger.debug("mv_cmd "+mv_cmd)
                    os.popen(mv_cmd)

                    cp_cmd="cp "+caccount_dir+logindata_file_in_chrome_name+" "+chrom_dir
                    logger.debug("cp_cmd "+cp_cmd)
                    os.popen(cp_cmd)

                    chown_cmd="chown "+username+":"+username+"  "+chrom_dir+logindata_file_in_chrome_name
                    logger.debug("chown_cmd "+chown_cmd)
                    os.popen(chown_cmd)
                else:
                    logger.debug("server logindata equals local,do nothing")
    else:
        logger.debug("no local logindata file,apply server logindata to local")
        mv_cmd="mv "+chrom_dir+logindata_file_in_chrome_name+" "+chrom_dir+logindata_file_in_chrome_name+".bk"
        logger.debug("mv_cmd "+mv_cmd)
        os.popen(mv_cmd)

        cp_cmd="cp "+caccount_dir+logindata_file_in_chrome_name+" "+chrom_dir
        logger.debug("cp_cmd "+cp_cmd)
        os.popen(cp_cmd)

        chown_cmd="chown "+username+":"+username+"  "+chrom_dir+logindata_file_in_chrome_name
        logger.debug("chown_cmd "+chown_cmd)
        os.popen(chown_cmd)   
    
    return username
    
#废弃
def comparePasswordDiscard(username):
    chrom_dir = get_chrom_dir(username)
    caccount_dir = get_caccount_dir(username)
    password_file_name = "Login Data"
    server_file_end = ".server"
    password_file_in_chrome_name = "Login\ Data"
   
    cp_cmd = 'cp '+chrom_dir+password_file_in_chrome_name+' '+caccount_dir
    os.popen(cp_cmd)

    nfshttpclient.download(username,password_file_name,caccount_dir+password_file_name+server_file_end)
    chrom_file_size = os.path.getsize(caccount_dir+password_file_name)
    server_file_size = os.path.getsize(caccount_dir+password_file_name+server_file_end)
    if server_file_size == 0:
        nfshttpclient.upload(username,password_file_name,caccount_dir+password_file_name)
    if server_file_size>chrom_file_size:
        cp_cmd="cp "+caccount_dir+password_file_name+server_file_end+" "+chrom_dir
        os.popen(cp_cmd)
        mv_cmd="mv "+chrom_dir+password_file_name+server_file_end+" "+chrom_dir+password_file_name
        os.popen(mv_cmd)
        chown_cmd="chown "+username+":"+username+"  "+chrom_dir+password_file_name
        os.popen(chown_cmd)
    else:
        nfshttpclient.upload(username,password_file_name,caccount_dir+password_file_name)
    return username
