#!/usr/bin/env python3

from __future__ import print_function
from gi.repository import GLib
import dbus
import dbus.service
import dbus.mainloop.glib
import json
import nfshttpclient;
usage = """Usage:
python3 xxx-service.py &
python3 xxx-client.py
python3 xxx-async-client.py
python3 xxx-client.py --exit-service
"""

"""
DemoException
"""
class DemoException(dbus.DBusException):
    _dbus_error_name = 'com.nfs.cloud.sync.account.DemoException'
"""
NfsDbusObject 
nfs cloud account sync
this is dbus server
"""
class NfsDbusObject(dbus.service.Object):

    @dbus.service.method("com.nfs.cloud.sync.AccountInterface",
                         in_signature='s', out_signature='s')
    def GetUserInfo(self, sys_user_name):
        print("GetUserInfo sys_user_name is :", str(sys_user_name))
        #根据用户名 查找云账户配置 根据是否登陆 未登陆返回空 已登陆，返回用户信息
        # return ""
        user_info = nfshttpclient.get_user_info(sys_user_name)
        if user_info:
            chrom_obj ={}
            chrom_obj['account_id']=user_info['phonenumber']
            chrom_obj['email']=user_info['email']
            chrom_obj['full_name']=user_info['nickName']
            chrom_obj['given_name']=user_info['userName']
            chrom_obj['avatar']=user_info['avatar']
            chrom_obj['is_sync_feature_enabled']=nfshttpclient.is_sync_feature_enabled(sys_user_name)
            return json.dumps(chrom_obj)
        else:
            return ""
        # return "{\"account_id\": \"13333\", \"email\": \"xuhy@nfschina.com\",\"full_name\": \" send2ocean\",\"given_name\": \" ocean\",\"is_sync_feature_enabled\":true}"

    @dbus.service.method("com.nfs.cloud.sync.AccountInterface",
                         in_signature='s', out_signature='s')
    def GetSyncAllDataTypesInfo(self, sys_user_name):
        sync_switch_info = nfshttpclient.get_sync_data_types_info(sys_user_name)
        if sync_switch_info:
            return sync_switch_info
        else:
            return ""

    @dbus.service.method("com.nfs.cloud.sync.AccountInterface",
                         in_signature='ss', out_signature='s')
    def UpdateSyncDataTypesInfo(self, sys_user_name,sync_config_json_str):
        print("UpdateSyncDataTypesInfo sys_user_name is :", str(sys_user_name))
        print("sync_config_json_str is :", str(sync_config_json_str))
        return nfshttpclient.update_sync_data_types_info(sys_user_name,sync_config_json_str)
        #根据用户名  获取同步项目
        # return "{\"syncAllDataTypes\": false, \"bookmarks\": true,\"passwords\":true}"
    
    @dbus.service.method("com.nfs.cloud.sync.AccountInterface",
                         in_signature='s', out_signature='s')
    def HandleSignout(self, sys_user_name):
        print("HandleSignout sys_user_name is :", str(sys_user_name))
        #close sync
        nfshttpclient.setSyncFeatureEnabled(sys_user_name,"false")
        return "success"

    @dbus.service.method("com.nfs.cloud.sync.AccountInterface",
                         in_signature='s', out_signature='s')
    def SetNfsCloudSyncFeatureDisable(self, sys_user_name):
        print("SetNfsCloudSyncFeatureDisable sys_user_name is :", str(sys_user_name))
        #close sync
        nfshttpclient.setSyncFeatureEnabled(sys_user_name,"false")
        return "disable success"

    @dbus.service.method("com.nfs.cloud.sync.AccountInterface",
                         in_signature='s', out_signature='s')
    def SetNfsCloudSyncFeatureEnable(self, sys_user_name):
        print("SetNfsCloudSyncFeatureEnable sys_user_name is :", str(sys_user_name))
        #enable sync
        nfshttpclient.setSyncFeatureEnabled(sys_user_name,"true")
        return "enable success"


    @dbus.service.method("com.nfs.cloud.sync.AccountInterface",
                         in_signature='s', out_signature='s')
    def UpdateBookmarkFile(self, sys_user_name):
        print("UpdateBookmarkFile sys_user_name is :", str(sys_user_name))
        bookmakr_file_name = "Bookmarks"
        chrome_bookmark_file = '/home/'+sys_user_name+'/.config/nfs-browser/Default/'+bookmakr_file_name
        nfshttpclient.upload(sys_user_name,bookmakr_file_name,chrome_bookmark_file)
        return "Update Bookmark File over"

    @dbus.service.method("com.nfs.cloud.sync.AccountInterface",
                         in_signature='', out_signature='')
    def RaiseException(self):
        raise DemoException('The RaiseException method does what you might '
                            'expect')
 

    @dbus.service.method("com.nfs.cloud.sync.AccountInterface",
                         in_signature='', out_signature='')
    def Exit(self):
        mainloop.quit()


if __name__ == '__main__':
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

    # session_bus = dbus.SessionBus()
    session_bus = dbus.SystemBus()
    name = dbus.service.BusName("com.nfs.cloud.sync.AccountService", session_bus)
    object = NfsDbusObject(session_bus, '/Objects')

    mainloop = GLib.MainLoop()
    print("Running nfs cloud syn dbus service.")
    print(usage)
    mainloop.run()