import json
import requests
import urllib3
import os
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
# HOST_BASE_URL = "http://40.40.40.60:8080/"
# HOST_BASE_URL = "http://172.30.98.66:8080/"
HOST_BASE_URL = "https://cloudaccount.os.nfschina.com/caccount-api/"

def getUserToken(username):
    configDir = "/home/"+username+"/.caccount/config"
    tokenJson = configDir+"/token.json"
    if os.path.isfile(tokenJson):
        tokenJsonFile = open(tokenJson, 'r')
        token = tokenJsonFile.read()
        tokenJsonFile.close()
        return token.strip('\n')
    else:
        return ""

def get_user_info(sys_user_name):
    token_str = getUserToken(sys_user_name)
    if token_str:
        local_user_info_json_file = get_local_user_info_config_file(sys_user_name)
        if os.path.isfile(local_user_info_json_file):
            chrome_f = open(local_user_info_json_file,'r')
            config = chrome_f.read()
            chrome_f.close()
            user_info_json = json.loads(config)
            print('read user json from local')
            return user_info_json
        else:
            info_url = HOST_BASE_URL+"getInfo"
            bear_token_str = "Bearer "+token_str
            headers_value = {"Authorization":bear_token_str}
            response = requests.get(info_url,headers=headers_value)
            return_json = response.json()
            print(return_json['user'] )
            chrome_f = open(local_user_info_json_file,'w')
            chrome_f.write(json.dumps(return_json['user']))
            chrome_f.close()
            print('write  user info to local json file')
            return return_json['user']
    else:
        return ""

def get_sync_data_types_info(sys_user_name):
    token_str = getUserToken(sys_user_name)
    if token_str:
        local_chrome_json_file = get_chrome_config_file(sys_user_name)
        if os.path.isfile(local_chrome_json_file):
            print("local chrom json file is exist")
            chrome_f = open(local_chrome_json_file,'r')
            config = chrome_f.read()
            chrome_f.close()
            config_json = json.loads(config)
            print('read chrom json')
            return config_json['syncItemSwitchConfig']
        else:
            info_url = HOST_BASE_URL+"business/chrome/config"
            bear_token_str = "Bearer "+token_str
            headers_value = {"Authorization":bear_token_str}
            response = requests.get(info_url,headers=headers_value)
            return_json = response.json()
            chrome_f = open(local_chrome_json_file,'w')
            chrome_f.write(json.dumps(return_json['data']))
            chrome_f.close()
            print('write to local chrom json')
            return return_json['data']['syncItemSwitchConfig']
    else:
        return ""

def update_sync_data_types_info(sys_user_name,sync_config_json_str):
    token_str = getUserToken(sys_user_name)
    if token_str:
        info_url = HOST_BASE_URL+"business/chrome/config"
        bear_token_str = "Bearer "+token_str
        headers_value = {"Authorization":bear_token_str}
        data_values={"syncItemSwitchConfig":sync_config_json_str}
        response = requests.post(info_url,headers=headers_value,data=data_values)
        return_json = response.json()
        local_chrome_json_file = get_chrome_config_file(sys_user_name)
        # if os.path.isfile(local_chrome_json_file):
        chrome_f = open(local_chrome_json_file,'r')
        config = chrome_f.read()
        chrome_f.close()
        config_json = json.loads(config)
        config_json['syncItemSwitchConfig']=sync_config_json_str
        chrome_f = open(local_chrome_json_file,'w')
        chrome_f.write(json.dumps(config_json))
        chrome_f.close()
        return return_json['msg']
    else:
        return ""
 
def get_local_user_info_config_file(sys_user_name):
    return "/home/"+sys_user_name+"/.caccount/config/user.json"
def get_chrome_config_file(sys_user_name):
    return "/home/"+sys_user_name+"/.caccount/config/chrome.json"

def setSyncFeatureEnabled(sys_user_name,isSyncFeatureEnabled):
    token_str = getUserToken(sys_user_name)
    if token_str:
        info_url = HOST_BASE_URL+"business/chrome/config"
        bear_token_str = "Bearer "+token_str
        headers_value = {"Authorization":bear_token_str}
        data_values={"isSyncFeatureEnabled":isSyncFeatureEnabled}
        response = requests.post(info_url,headers=headers_value,data=data_values)
        return_json = response.json()
        local_chrome_json_file = get_chrome_config_file(sys_user_name)
        chrome_f = open(local_chrome_json_file,'w')
        chrome_f.write(json.dumps(return_json['data']))
        chrome_f.close()
        print("setSyncFeatureEnabled return is ")
        print(return_json)
        return return_json['msg']
    else:
        return ""

def is_sync_feature_enabled(sys_user_name):
    local_chrome_json_file = get_chrome_config_file(sys_user_name)
    if os.path.isfile(local_chrome_json_file):
        print("is_sync_feature_enabled, local_chrome_json_file  is  exist")
    else:
        get_sync_data_types_info(sys_user_name)
    chrom_json_file = open(local_chrome_json_file, 'r')
    config = chrom_json_file.read()
    chrom_json_file.close()
    config =  config.strip('\n')
    config_json = json.loads(config)
    if config_json['isSyncFeatureEnabled']:
        if config_json['isSyncFeatureEnabled']=='true':
            return True
        else:
            return False
    else:
        return False

def download(sys_user_name,fileName,destFile):
    
    token_str = getUserToken(sys_user_name)
    if token_str:
        info_url = HOST_BASE_URL+"business/chrome/download"
        bear_token_str = "Bearer "+token_str
        headers_value = {"Authorization":bear_token_str}
        params_values={"fileName":fileName}
        response = requests.get(info_url,headers=headers_value,params=params_values,stream=True)
        f = open(destFile,"wb")
        for chunk in response.iter_content(chunk_size=512):
            if chunk:
                f.write(chunk)
        f.close()
    else:
        return ""


def upload(sys_user_name,fileName,sourceFile):

    token_str = getUserToken(sys_user_name)
    if token_str:
        info_url = HOST_BASE_URL+"business/chrome/upload"
        bear_token_str = "Bearer "+token_str
        headers_value = {"Authorization":bear_token_str}
        files = {'file':open(sourceFile,'rb')}
        data_values={"fileName":fileName}
        response = requests.post(info_url,headers=headers_value,data=data_values,files=files)
        return_json = response.json()
        print("updload file to server,return is ")
        print(return_json)
    else:
        return ""